package soal.ppr.experiments

import java.util.Random

import com.twitter.cassovary.graph.{Node, DirectedGraph}
import it.unimi.dsi.fastutil.floats.FloatArrayList
import soal.ppr.BidirectionalPPREstimator

import scala.collection.JavaConverters._

class ReversePushExperiments(graph: DirectedGraph[Node], teleportProb: Float) {
  val random = new Random()
  val pprEstimator = new BidirectionalPPREstimator(graph, teleportProb, random)

  def residualDistributionExperiment(pairCount: Int): Unit = {
    val maxResidual = 10.0f / math.sqrt(graph.edgeCount).toFloat
    println(s"maxResidual = $maxResidual")
    val allResiduals = new FloatArrayList(1000 * 1000)
    for (pairIndex <- 0 until pairCount) {
      val target = ExperimentUtils.sampleFromPageRank(graph, teleportProb)

      val (estimates, residuals) = ExperimentUtils.printTime("computeContributions took ") {
        pprEstimator.computeContributions(target.id, maxResidual)
      }
      residuals.values foreach (allResiduals.add(_))
      if ((pairIndex + 1) % (pairCount / 10) == 0)
        println("residuals = " + ExperimentUtils.toPythonList(allResiduals.asScala))
    }
    println(s"maxResidual = $maxResidual")
    println("residuals = " + ExperimentUtils.toPythonList(allResiduals.asScala))
  }
}