package soal.ppr.experiments

import com.twitter.cassovary.graph.{Node, DirectedGraph}
import soal.ppr.{BidirectionalPPRPrecomputor, BidirectionalPPREstimator}


object StorageExperiments {
  def storageSizesVaryingResidual(graph: DirectedGraph[Node], teleportProbability: Double,
                                  nodeCount: Int) {

    val nodeIds = 0 until graph.maxNodeId // Assume ids are sequential
    val targetNodeIds = 0 until nodeCount  map { x => ExperimentUtils.randomElement(nodeIds) } filter { graph.getNodeById(_).nonEmpty }
    val startIds = 0 until nodeCount  map { x => ExperimentUtils.randomElement(nodeIds) } filter { graph.getNodeById(_).nonEmpty }
    val pprThreshold = 1.0f / graph.nodeCount
    val chernoffConstant = 7
    val reverseThresholds = for (i <- -9 to 9) yield
      (math.sqrt(pprThreshold) * math.pow(2.0, i / 3.0)).toFloat
    val estimator = new BidirectionalPPREstimator(graph, teleportProbability.toFloat)
    val precomputor = new BidirectionalPPRPrecomputor(graph, teleportProbability.toFloat)

    val meanReverseSizes = for (reverseThreshold <- reverseThresholds) yield {
      println(s"\n\n----------reverseThreshold $reverseThreshold -------------")
      val sizes = for (targetId <- targetNodeIds) yield {
          val (estimates, residuals) = estimator.computeContributions(targetId, reverseThreshold)
          val size = estimates.size + residuals.size
          println("  node " + targetId + " has (reverse) frontier size " + size)
          size.toFloat
      }
      sizes.mkString("[", ",", "]")
      ExperimentUtils.mean(sizes)
    }
    println(s"\n\n----------Finished reverse, now measuring forward -------------")

    val meanForwardSizes = for (reverseThreshold <- reverseThresholds) yield {
      println(s"\n\n----------reverseThreshold $reverseThreshold -------------")
      val sizes = for (s <- startIds) yield {
          val walkDistribution = precomputor.forwardPrecomputation(s, pprThreshold,
            reverseThreshold)
          val size = walkDistribution.size
          println(s"  node $s has walkDistribution size $size")
          size.toFloat
        }
      sizes.mkString("[", ",", "]")
      ExperimentUtils.mean(sizes)
    }

    println("reverseThresholds = " +reverseThresholds.mkString("[", ",", "]"))
    println("meanReverseSizes = " + meanReverseSizes.mkString("[", ",", "]"))
    println("meanForwardSizes = " + meanForwardSizes.mkString("[", ",", "]"))
  }
}
