package soal.util
import collection._
import net.openhft.koloboke.collect.map.hash.{HashIntDoubleMaps, HashIntFloatMaps}
import scala.collection.JavaConverters._

/**
 * Various useful functions on collections.
 */

object CollectionsUtil {
  /**
   * Returns a map whose value on key k is equal to the sum of the values of the given maps.
   */
  def sumMaps(maps: Traversable[Map[Int, Float]]): mutable.Map[Int, Float] = {
    val totals = CollectionsUtil.efficientIntFloatMapWithDefault0()
    for (map <- maps;
         (k, v) <- map) {
      totals(k) += v
    }
    totals
  }

  def dotMaps(xs: Map[Int, Float], ys: Map[Int, Float]): Float = {
    (xs map { case (key, x) =>
      x * ys.getOrElse(key, 0.0f)
    }).sum
  }

  /**
   * Adds ys(i) to xs(i), storing the result in xs.
   */
  def plusEquals(xs: mutable.IndexedSeq[Double], ys: IndexedSeq[Double]): Unit = {
    for ((yi, i) <- ys.zipWithIndex) {
      xs(i) += yi
    }
  }

  /**
   * Multiplies each element of m by c.
   */
  def scale(m: Seq[Seq[Double]], c: Double): Seq[Seq[Double]] = {
    m map { row =>
      row map { _ * c }
    }
  }

  def scaleSeq(xs: Seq[Double], c: Double): Seq[Double] = {
    xs map (_ * c)
  }

  /**
   * Multiplies each element of m by c.
   */
  def scaleMatrix(m: Array[Array[Double]], c: Double): Array[Array[Double]] = {
    m map { row =>
      row map { _ * c }
    }
  }

  /**
   * Returns an efficient map, so if we change the primative map library we use we can just
   * change this line of code.
   */
  def efficientIntFloatMap(): mutable.Map[Int, Float] =
    // For some reason, asInstanceOf is still necessary to convert java.lang.Integer to scala.Int
    HashIntFloatMaps.newMutableMap().asScala.asInstanceOf[mutable.Map[Int, Float]]
  // In the future, this might be replaced with a more efficient version
  def efficientIntFloatMapWithDefault0(): mutable.Map[Int, Float] =
    efficientIntFloatMap().withDefaultValue(0.0f)
  def efficientIntDoubleMap(): mutable.Map[Int, Double] =
  // For some reason, asInstanceOf is still necessary to convert java.lang.Integer to scala.Int
    HashIntDoubleMaps.newMutableMap().asScala.asInstanceOf[mutable.Map[Int, Double]]
  def efficientIntDoubleMapWithDefault0(): mutable.Map[Int, Double] =
    efficientIntDoubleMap().withDefaultValue(0.0)

}
