package soal.ppr

import java.util.Random

import org.scalatest.{FlatSpec, Matchers}
import soal.util.{ConstantDistribution, CassovaryUtil}

/**
 * Created by peter on 9/17/15.
 */
class BidirectionalUndirectedPPREstimatorSpec extends FlatSpec with Matchers {
  val graph = CassovaryUtil.readGraph("src/test/resources/test_graph.txt")
  val teleportProb = 0.2f
  val random = new Random(2) // Seed for consistent tests
  val estimator = new BidirectionalUndirectedPPREstimator(graph, teleportProb, random)
  val directedEstimator = new BidirectionalPPREstimator(graph, teleportProb, random)

  "BidirectionalPPRSearcher.estimatePPR" should "be correct on the test graph" in {
    val relativeError = 0.01f
    val minPPR = 0.03f
    val stPairs = Array(0 -> 1, 2 -> 3, 5 -> 9, 0 -> 0)

    for ((s, t) <- stPairs) {
      withClue (s"Testing Pair ($s, $t)") {
        val truePPR = directedEstimator.estimatePPR(new ConstantDistribution(s),
          t, minPPR, relativeError, guaranteeRelativeError = true)
        estimator.estimatePPR(s, t, minPPR, relativeError, guaranteeRelativeError = true) should equal (
          truePPR +- truePPR * relativeError * 2)
      }
    }
  }
}
